//
// * Licensed Materials - Property of IBM
// * 
// * 5724-H72, 5655-R36, 5724-L26, 5655-L82     
// * 
// * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.
// * 
// * US Government Users Restricted Rights - Use, duplication or
// * disclosure restricted by GSA ADP Schedule Contract with
// * IBM Corp.
package com.ibm.scenario.wmq.jms;


import java.io.PrintStream;
import java.lang.reflect.Field;

import com.ibm.mq.MQException;

/**

 */
public class ExceptionHandling {

  static int titleSize = 30;
  private static String lineSeparator;

  static {
    lineSeparator = System.getProperty("line.separator");
  }

  private static String padTitle(String text) {
    StringBuffer buffer = new StringBuffer(titleSize);
    int textLength = text.trim().length();

    int padding = titleSize - textLength;
    for (int count = 0; count < padding; count++) {
      buffer.append(" ");
    }

    buffer.append(text.trim()).append(" : ");

    return buffer.toString();

  }

  public static void handleException(final Exception exception) {
    System.out
        .println("------------------------------HANDLING EXCEPTION------------------------------");
    PrintStream stream = System.out;

    int causeIndex = 0;
    Throwable loopException = exception;

    while (loopException != null) {
      augmentStackTrace(loopException);
      
      if (causeIndex == 0) {
        stream.println(padTitle("Message") + loopException.getMessage());
      }
      else {
        stream.println(padTitle("Caused by [" + causeIndex + "] --> Message")
            + loopException.getMessage());
      }

      stream.println(padTitle("Class") + loopException.getClass());

      if (loopException instanceof MQException) {
        final MQException mqException = (MQException) loopException;
        stream.println(padTitle("Comp code") + mqException.completionCode);
        stream.println(padTitle("Reason code") + mqException.reasonCode);
      }

      dumpStackTrace(loopException, stream);

      loopException = loopException.getCause();
      causeIndex++;
    }
    System.out
        .println("------------------------------HANDLED EXCEPTION------------------------------");
  }

  private static void dumpStackTrace(final Throwable e, PrintStream stream) {
    StackTraceElement stack[] = e.getStackTrace();
    for (int count = 0; count < stack.length; count++) {
      String prefix;
      if (count == 0) {
        prefix = "Stack";
      }
      else {
        prefix = "";
      }

      String className = stack[count].getClassName();
      String methodName = stack[count].getMethodName();
      String fileName = stack[count].getFileName();
      int line = stack[count].getLineNumber();

      stream.println(padTitle(prefix) + className + "." + methodName + "(" + fileName + ":" + line
          + ")");
    }
  }
  
  
  public static void augmentStackTrace(Throwable t){
    
//    System.out.println("Before Augmentation.... "+ t);
//    t.printStackTrace(System.out);
//    System.out.println("-------------------------");
    
    StackTraceElement elements[] = t.getStackTrace();
    for (int i=0; i<elements.length;i++){
      StackTraceElement element = elements[i];
      String className = element.getClassName();
//      System.out.println("[Got classname "+className+"]");
      
      try {
        Class c = Class.forName(className);
        Field fields[] = c.getDeclaredFields();
        Field field=null;
        for (int f=0; f<fields.length;f++){
          field = fields[f];
          if (field.getName().startsWith("sccs")){
            break;
          }
        }
        if (field==null){
          continue;
        }
        //System.out.println("Found field "+field);
        field.setAccessible(true);
        //System.out.println(field.get(null));
        
        // parse out the SCCS ID required in this instance
        Object info = field.get(null);
        String sccs;
        if (info!=null){
           String infos[] = info.toString().split(" ");
           sccs = "["+infos[4]+" "+infos[5]+"](";
        }else  {
          sccs = "[n.a]";
        }
        elements[i] = new StackTraceElement(element.getClassName(),element.getMethodName(),sccs+element.getFileName(),element.getLineNumber());
        
        
      } catch (Exception e1){
//        System.out.println(e1+" @ "+element); 
//        e1.printStackTrace();
      }
      
      t.setStackTrace(elements);
    }
       
    
  }

}
